<?php

declare(strict_types=1);

namespace Drupal\views_access_conditions;

use Drupal\conditions_helper\ConditionsEvaluator;
use Drupal\conditions_helper\Form\ConditionsFormBuilder;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\Context\ContextRepositoryInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\views\ViewExecutable;
use Drupal\views_access_conditions\Form\SettingsForm;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Constructs a new \Drupal\views_access_conditions\ViewsAlters object.
 */
final class ViewsAlters implements ContainerInjectionInterface {

  use DependencySerializationTrait;
  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $module_handler = $container->get('module_handler');
    $entity_field_manager = $container->get('entity_field.manager');
    $context_repository = $container->get('context.repository');
    $entity_type_manager = $container->get('entity_type.manager');
    $conditions_helper_form_builder = $container->get('conditions_helper.form_builder');
    $conditions_evaluator = $container->get('conditions_helper.evaluator');

    assert($module_handler instanceof ModuleHandlerInterface);
    assert($entity_field_manager instanceof EntityFieldManagerInterface);
    assert($context_repository instanceof ContextRepositoryInterface);
    assert($entity_type_manager instanceof EntityTypeManagerInterface);
    assert($conditions_helper_form_builder instanceof ConditionsFormBuilder);
    assert($conditions_evaluator instanceof ConditionsEvaluator);

    return new self(
      $module_handler,
      $entity_field_manager,
      $context_repository,
      $entity_type_manager,
      $conditions_helper_form_builder,
      $conditions_evaluator,
    );
  }

  /**
   * Constructor for FormAlters.
   */
  public function __construct(
    protected readonly ModuleHandlerInterface $moduleHandler,
    protected readonly EntityFieldManagerInterface $entityFieldManager,
    protected readonly ContextRepositoryInterface $contextRepository,
    protected readonly EntityTypeManagerInterface $entityTypeManager,
    protected readonly ConditionsFormBuilder $conditionsHelperFormBuilder,
    protected readonly ConditionsEvaluator $conditionsEvaluator,
  ) {
  }

  /**
   * Alter the views config item forms.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function formAlter(array &$form, FormStateInterface $form_state): void {
    $display_id = $form_state->get('display_id');
    $type = $form_state->get('type');
    $form['options']['third_party_settings']['views_access_conditions'] = [
      '#type' => 'details',
      '#title' => $this->t('Views Access Conditions'),
      '#open' => TRUE,
      '#tree' => TRUE,
    ];

    $view = $form_state->get('view');
    $third_party_settings = $view->getThirdPartySettings('views_access_conditions') ?? [];

    $available_conditions = $this->conditionsHelperFormBuilder->getAvailableConditions(SettingsForm::SETTINGS);
    $available_contexts = $this->contextRepository->getAvailableContexts();
    $stored_values = $third_party_settings[$display_id][$type][$form_state->get('id')] ?? [];
    $this->conditionsHelperFormBuilder->buildConditionsForm($form['options']['third_party_settings']['views_access_conditions'], $form_state, $available_conditions, $available_contexts, $stored_values);

    // Add our submission handler to be first.
    array_unshift(
      $form['actions']['submit']['#submit'],
      [$this, 'argumentFormSubmit']
    );
  }

  /**
   * Submit callback for the form being submitted.
   *
   * @param array $form
   *   The form being altered.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function argumentFormSubmit(array &$form, FormStateInterface $form_state): void {
    $display_id = $form_state->get('display_id');
    $type = $form_state->get('type');

    $this->conditionsHelperFormBuilder->submitConditionsForm(
      $form,
      $form_state,
      ['options', 'third_party_settings', 'views_access_conditions']
    );

    $submitted_third_party_settings = $form_state->getValue([
      'options',
      'third_party_settings',
      'views_access_conditions',
    ]) ?? [];

    if (!empty($submitted_third_party_settings)) {
      $view = $form_state->get('view');
      $views_third_party_settings = $view->getThirdPartySettings('views_access_conditions') ?? [];
      $views_third_party_settings[$display_id][$type][$form_state->get('id')] = $submitted_third_party_settings;
      $view->setThirdPartySetting('views_access_conditions', $display_id, $views_third_party_settings[$display_id]);
    }
  }

  /**
   * Alter the exposed form.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function exposedFormAlter(array &$form, FormStateInterface $form_state): void {
    // Get the display id and the view object.
    $display_id = $form_state->get('display')['id'] ?? '';
    $view = $form_state->get('view');
    /** @var \Drupal\views\Entity\ViewInterface $view_config */
    $view_config = $this->entityTypeManager->getStorage('view')->load($view->id());

    if (empty($display_id) || empty($view_config)) {
      return;
    }

    // Get the third party settings for this view.
    $third_party_settings = $view_config->getThirdPartySettings('views_access_conditions') ?? [];
    if (empty($third_party_settings[$display_id])) {
      return;
    }

    // Get the third party settings for this display.
    $settings = $third_party_settings[$display_id];

    // Walk through the form and alter the access based on the conditions.
    array_walk($form, function ($value, $key) use (&$form, $settings) {
      if (str_starts_with($key, '#')) {
        return;
      }

      // If the filter has conditions, evaluate them.
      if (isset($settings['filter'][$key])) {
        $result = $this->conditionsEvaluator->evaluateConditions($settings['filter'][$key]);
        if ($result === FALSE) {
          $form[$key]['#access'] = FALSE;
        }
      }
    });
  }

  /**
   * Alter the pre-build of the view.
   *
   * @param \Drupal\views\ViewExecutable $view
   *   The view executable.
   */
  public function preBuildAlter(ViewExecutable $view): void {
    // Get the view config and the third party settings.
    /** @var \Drupal\views\Entity\ViewInterface $view_config */
    $view_config = $this->entityTypeManager->getStorage('view')->load($view->id());
    $third_party_settings = $view_config->getThirdPartySettings('views_access_conditions') ?? [];
    if (empty($third_party_settings)) {
      return;
    }

    // Get the display id.
    $display_id = $view->current_display;

    if (empty($third_party_settings[$display_id])) {
      return;
    }

    // Get the third party settings for this display.
    $settings = $third_party_settings[$display_id];

    // Walk through the third party settings configured for this display.
    array_walk($settings, function ($plugin_settings, $plugin_type) use (&$view) {
      // There can be conditions set on different types of plugins, for example
      // filters, fields, etc. Find the specific plugin type in the view object,
      // and walk through the fields and check if the conditions are met.
      if (!empty($view->{$plugin_type})) {
        $i = 0;
        array_walk($view->{$plugin_type}, function ($plugin, $key) use (&$view, $plugin_settings, $plugin_type, &$i) {
          // If the field plugin has conditions, evaluate them.
          if (isset($plugin_settings[$key])) {
            $result = $this->conditionsEvaluator->evaluateConditions($plugin_settings[$key]);
            if ($result === FALSE) {
              unset($view->{$plugin_type}[$key]);
              if ($plugin_type === 'argument') {
                unset($view->args[$i]);
              }
            }
          }
          $i++;
        });
      }
    });

  }

}
